class PromiseResolver {
  constructor(){
    var this_ = this;
    this.promise = new Promise((resolve, reject) => {
      this_.resolve_func = resolve;
      this_.reject_func = resolve;
    })
  }
  getPromise(){
    return this.promise;
  }
  resolveWithResult(result){
    this.resolve_func(result);
  }
  rejectWithError(err){
    this.reject_func(err);
  }
}

class Messenger {
  constructor(local, remote){
    var this_ = this;
    this.local = local;
    this.remote = remote;
    this.messages = [];
    this.requests = [];
    local.onmessage = function(event) {
      this_.messages.push(event.data);
      if(this_.requests.length != 0){
        var request = this_.requests.shift();
        request.resolveWithResult(this_.messages.shift());
      }
    }
  }
  sendMessage(msg){
    this.remote.postMessage(msg);
  }
  async receiveMessage(){
    if(this.messages.length == 0){
      var resolver = new PromiseResolver();
      this.requests.push(resolver);
      return resolver.getPromise();
    } else {
      return this.messages.shift();
    }
  }
}

function sleep(ms) {
  return new Promise((resolve, reject) => {
    setTimeout(resolve, ms);
  })
}

async function bailout(ms) {
  await sleep(ms);
  window.location.reload();
}


function logToHTML(data) {
  var node = document.getElementById("log_div");
  var inner_html = node.innerHTML;
  node.innerHTML = inner_html + data + "<br>";
  console.log(data);
}

async function registerWithUrlAndScope(url, scope) {
  return navigator.serviceWorker.register(url, { 'scope': scope });
}

async function activeServiceWorker(url, scope){
  var registration = await registerWithUrlAndScope(url, scope);
  promise_resolver = new PromiseResolver();
  var reg = await navigator.serviceWorker.ready;
  reg.active.addEventListener('statechange', async (event) => {
    if(event.target.state == "activated"){
      promise_resolver.resolveWithResult(registration);
    }
  })
  await promise_resolver.getPromise();
}

function gc() {
  for (var i = 0; i < 0x500; i++) new Array(0x1000);
}